/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.hibernate;

import java.io.Serializable;
import java.sql.Connection;

import org.hibernate.CacheMode;
import org.hibernate.Criteria;
import org.hibernate.EntityMode;
import org.hibernate.FlushMode;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.ReplicationMode;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.stat.SessionStatistics;
import org.hibernate.Filter;

import it.eng.spago.base.RequestContainer;

public abstract class AbstractHibernateSession implements Session {
    private Session _nativeSession = null;

    public AbstractHibernateSession(Session nativeSession) {
        super();
        _nativeSession = nativeSession;
    } // public HibernateSession(Session nativeSession)

    public EntityMode getEntityMode(){
    	return _nativeSession.getEntityMode();
    }

    public Session getSession(EntityMode entitymode){
    	return _nativeSession.getSession(entitymode);
    }

    public void flush() throws HibernateException {
        _nativeSession.flush();
    } //public void flush() throws HibernateException 

    public void setFlushMode(FlushMode flushMode) {
        _nativeSession.setFlushMode(flushMode);
    } // public void setFlushMode(FlushMode flushMode)

    public FlushMode getFlushMode() {
        return _nativeSession.getFlushMode();
    } // public FlushMode getFlushMode()

    public void setCacheMode(CacheMode cachemode){
    	_nativeSession.setCacheMode(cachemode);
    }

    public CacheMode getCacheMode(){
    	return _nativeSession.getCacheMode();
    }

    public SessionFactory getSessionFactory() {
        return _nativeSession.getSessionFactory();
    } // public SessionFactory getSessionFactory()     

    public Object load(String s, Serializable serializable, LockMode lockmode)
        throws HibernateException {
    	return _nativeSession.load(s, serializable, lockmode);
    }

    public Object load(String s, Serializable serializable)
        throws HibernateException {
    	return _nativeSession.load(s, serializable);
    }

    public Serializable save(String s, Object obj)
        throws HibernateException {
    	return _nativeSession.save(s, obj);
    }

    public void saveOrUpdate(String s, Object obj)
        throws HibernateException {
    	_nativeSession.saveOrUpdate(s, obj);
    }

    public void update(String s, Object obj)
        throws HibernateException {
    	_nativeSession.update(s, obj);
    }

    public Object merge(Object obj)
        throws HibernateException {
    	return _nativeSession.merge(obj);
    }

    public Object merge(String s, Object obj)
        throws HibernateException {
    	return _nativeSession.merge(s, obj);
    }

    public void persist(Object obj)
        throws HibernateException {
    	_nativeSession.persist(obj);
    }

    public void persist(String s, Object obj)
        throws HibernateException {
    	_nativeSession.persist(s, obj);
    }

    public void lock(String s, Object obj, LockMode lockmode)
        throws HibernateException {
    	_nativeSession.lock(s, obj, lockmode);
    }

    public Transaction getTransaction() {
    	return _nativeSession.getTransaction();
    }

    public Criteria createCriteria(Class class1, String s) {
    	return _nativeSession.createCriteria(class1, s);
    }

    public Criteria createCriteria(String s) {
    	return _nativeSession.createCriteria(s);
    }

    public Criteria createCriteria(String s, String s1) {
    	return _nativeSession.createCriteria(s, s1);
    }

    public Object get(String s, Serializable serializable)
        throws HibernateException {
        	return _nativeSession.get(s, serializable);
    }

    public Object get(String s, Serializable serializable, LockMode lockmode)
        throws HibernateException {
        	return _nativeSession.get(s, serializable, lockmode);
        }

    public String getEntityName(Object obj) 
        throws HibernateException {
        	return _nativeSession.getEntityName(obj);
        }

    public Filter enableFilter(String s){
    	return _nativeSession.enableFilter(s);
    }

    public Filter getEnabledFilter(String s){
    	return _nativeSession.getEnabledFilter(s);
    }

    public void disableFilter(String s) {
    	_nativeSession.disableFilter(s);
    }

    public SessionStatistics getStatistics() {
    	return _nativeSession.getStatistics();
    }

    public void setReadOnly(Object obj, boolean flag) {
    	_nativeSession.setReadOnly(obj, flag);
	}
 
    public Connection connection() throws HibernateException {
        return _nativeSession.connection();
    } // public Connection connection() throws HibernateException

    public Connection disconnect() throws HibernateException {
        return _nativeSession.disconnect();
    } // public Connection disconnect() throws HibernateException

    public void reconnect() throws HibernateException {
        _nativeSession.reconnect();
    } // public void reconnect(Connection connection) throws HibernateException
    
    public void reconnect(Connection connection) throws HibernateException {
        _nativeSession.reconnect(connection);
    } // public void reconnect(Connection connection) throws HibernateException

    public Connection close() throws HibernateException {
        return _nativeSession.close();
    } // public Connection close() throws HibernateException

    public void cancelQuery() throws HibernateException {
        _nativeSession.cancelQuery();
    } // public void cancelQuery() throws HibernateException

    public boolean isOpen() {
        return _nativeSession.isOpen();
    } // public boolean isOpen()

    public boolean isConnected() {
        return _nativeSession.isConnected();
    } // public boolean isConnected()

    public boolean isDirty() throws HibernateException {
        return _nativeSession.isDirty();
    } // public boolean isDirty() throws HibernateException

    public Serializable getIdentifier(Object object)
        throws HibernateException {
        return _nativeSession.getIdentifier(object);
    } // public Serializable getIdentifier(Object object) throws HibernateException

    public boolean contains(Object object) {
        return _nativeSession.contains(object);
    } // public boolean contains(Object object)

    public void evict(Object object) throws HibernateException {
        _nativeSession.evict(object);
    } // public void evict(Object object) throws HibernateException

    public Object load(Class theClass, Serializable id, LockMode lockMode)
        throws HibernateException {
        return _nativeSession.load(theClass, id, lockMode);
    } // public Object load(Class theClass, Serializable id, LockMode lockMode) throws HibernateException

    public Object load(Class theClass, Serializable id)
        throws HibernateException {
        return _nativeSession.load(theClass, id);
    } // public Object load(Class theClass, Serializable id) throws HibernateException

    public void load(Object object, Serializable id)
        throws HibernateException {
        _nativeSession.load(object, id);
    } // public void load(Object object, Serializable id) throws HibernateException

    public void replicate(Object object, ReplicationMode replicationMode)
        throws HibernateException {
        _nativeSession.replicate(object, replicationMode);
    } // public void replicate(Object object, ReplicationMode replicationMode) throws HibernateException

    public void replicate(String s, Object object, ReplicationMode replicationMode)
    throws HibernateException {
    _nativeSession.replicate(s, object, replicationMode);
    } // public void replicate(Object object, ReplicationMode replicationMode) throws HibernateException
    
    public Serializable save(Object object)
        throws HibernateException {
        return _nativeSession.save(object);
    } // public void save(Object object, Serializable id) throws HibernateException

    public void saveOrUpdate(Object object) throws HibernateException {
        _nativeSession.saveOrUpdate(object);
    } // public void saveOrUpdate(Object object) throws HibernateException

    public void update(Object object) throws HibernateException {
        _nativeSession.update(object);
    } // public void update(Object object) throws HibernateException

    public void delete(Object object) throws HibernateException {
        _nativeSession.delete(object);
    } // public void delete(Object object) throws HibernateException

    public void delete(String entityName, Object object) throws HibernateException {
        _nativeSession.delete(object);
    } 

    public void lock(Object object, LockMode lockMode)
        throws HibernateException {
        _nativeSession.lock(object, lockMode);
    } // public void lock(Object object, LockMode lockMode) throws HibernateException

    public void refresh(Object object) throws HibernateException {
        _nativeSession.refresh(object);
    } // public void refresh(Object object) throws HibernateException

    public void refresh(Object object, LockMode lockMode)
        throws HibernateException {
        _nativeSession.refresh(object, lockMode);
    } // public void refresh(Object object, LockMode lockMode) throws HibernateException

    public LockMode getCurrentLockMode(Object object)
        throws HibernateException {
        return _nativeSession.getCurrentLockMode(object);
    } // public LockMode getCurrentLockMode(Object object) throws HibernateException

    public Transaction beginTransaction() throws HibernateException {
        return _nativeSession.beginTransaction();
    } // public Transaction beginTransaction() throws HibernateException

    public Criteria createCriteria(Class persistentClass) {
        return _nativeSession.createCriteria(persistentClass);
    } // public Criteria createCriteria(Class persistentClass)

    public Query createQuery(String queryString) throws HibernateException {
        return _nativeSession.createQuery(queryString);
    } // public Query createQuery(String queryString) throws HibernateException

    public Query createFilter(Object collection, String queryString)
        throws HibernateException {
        return _nativeSession.createFilter(collection, queryString);
    } // public Query createFilter(Object collection, String queryString) throws HibernateException

    public Query getNamedQuery(String queryName) throws HibernateException {
        return _nativeSession.getNamedQuery(queryName);
    } // public Query getNamedQuery(String queryName) throws HibernateException

    public SQLQuery createSQLQuery(String queryString) throws HibernateException {
        return _nativeSession.createSQLQuery(queryString);
    } // public Query createSQLQuery(String sql, String[] returnAliases, Class[] returnClasses)

    public void clear() {
        _nativeSession.clear();
    } // public void clear()

    public Object get(Class clazz, Serializable id) throws HibernateException {
        return _nativeSession.get(clazz, id);
    } // public Object get(Class clazz, Serializable id) throws HibernateException

    public Object get(Class clazz, Serializable id, LockMode lockMode)
        throws HibernateException {
        return _nativeSession.get(clazz, id, lockMode);
    } // public Object get(Class clazz, Serializable id, LockMode lockMode) throws HibernateException

    private boolean isRequestDistributed() {
        boolean distributedBoolean = false;
        RequestContainer requestContainer =
            RequestContainer.getRequestContainer();
        if (requestContainer != null)
            distributedBoolean = requestContainer.isRequestDistributed();
        return distributedBoolean;
    } // private boolean isRequestDistributed()

} // public abstract class AbstractHibernateSession implements Session
