package it.eng.console.audit.action;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.tracing.TracerSingleton;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.joda.time.DateTime;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.model.Process;

public class BAMGraphsAction extends AbstractAction {

	private static final long serialVersionUID = 1L;
	private static SimpleDateFormat sdffmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		// Produce the data to show in the graphs
		SourceBean sb = new SourceBean("PROCESSES");
		
		// Retrieve all processes
		Criteria criteria = ProcessBO.getProcessList();
		List<Process> processList = criteria.list();

		Date fromDate = calculateFromDate(serviceRequest, serviceResponse);
		//System.out.println(" ******************************** "+formatDate(fromDate) + "*******************************");
		for (Process process : processList) {
			SourceBean processSb = new SourceBean("PROCESS");
			
			
			if (fromDate != null){
				TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "From date "+ formatDate(fromDate));
			}else{
				TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "From date [NO TIME INTERVAL]");
			}
			
			long counter = ProcessInstanceBO.getCountProcessInstancesByState(process.getIdProcess(),ProcessInstanceStateConstants.PROCESS_DONE,fromDate);
			long counterFaulted = ProcessInstanceBO.getCountProcessInstancesByState(process.getIdProcess(), ProcessInstanceStateConstants.PROCESS_FAULTED, fromDate);
			long counterActive = ProcessInstanceBO.getCountProcessInstancesByState(process.getIdProcess(), ProcessInstanceStateConstants.PROCESS_ACTIVE, null);
			long counterWarning = ProcessInstanceBO.getCountProcessInstancesByState(process.getIdProcess(), ProcessInstanceStateConstants.PROCESS_WARNING, fromDate);
			double meanTime = 0;
			if (counter > 0){
				meanTime = ProcessInstanceBO.getMeanExecutionTime(process.getIdProcess(), ProcessInstanceStateConstants.PROCESS_DONE, fromDate);
			}
			
			
			processSb.setAttribute("name", process.getName());
			processSb.setAttribute("version", process.getProcessVersion());
			processSb.setAttribute("counter", String.valueOf(counter));
			processSb.setAttribute("counterFaulted", String.valueOf(counterFaulted));
			processSb.setAttribute("counterActive", String.valueOf(counterActive));
			processSb.setAttribute("counterWarning", String.valueOf(counterWarning)); 
			processSb.setAttribute("meanTime", meanTime);
			sb.setAttribute(processSb);
		}
		serviceResponse.setAttribute(sb);
	}

	public Date calculateFromDate(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception{
		String timeInterval = (String)serviceRequest.getAttribute("timeInterval");
		
		if (timeInterval != null &&  !timeInterval.equalsIgnoreCase("NO_TIME_INTERVAL")){
			DateTime dt = new DateTime();
			DateTime result = new DateTime();
			if (timeInterval.equals("1mm")){
				result =  dt.minusMinutes(1);
			}else if (timeInterval.equals("15mm")){
				result =  dt.minusMinutes(15);
			}else if (timeInterval.equals("30mm")){
				result =  dt.minusMinutes(30);
			}else if (timeInterval.equals("1hh")){
				result =  dt.minusHours(1);
			}else if (timeInterval.equals("6hh")){
				result =  dt.minusHours(6);
			}else if (timeInterval.equals("12hh")){
				result =  dt.minusHours(12);
			}else if (timeInterval.equals("1dd")){
				result =  dt.minusDays(1);
			}else if (timeInterval.equals("15dd")){
				result =  dt.minusDays(15);
			}else if (timeInterval.equals("1MM")){
				result =  dt.minusMonths(1);
			}
			serviceResponse.setAttribute("timeInterval",timeInterval );
			return result.toDate();
		}else{
			serviceResponse.setAttribute("timeInterval","NO_TIME_INTERVAL" );
			return null;
		}
		
	}
	
	public static String formatDate(Date date) {
    	if (date == null)
    		return "";
    	return sdffmt.format(date);
    }


}
