/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;

import java.rmi.RemoteException;

import org.dom4j.Document;
import org.dom4j.io.DOMReader;
import org.spagic.components.jobs.backup.BckDynDataStart_v_200Proxy;
import org.spagic.components.jobs.delete.DelDynDataStart_v_200Proxy;
import org.spagic.metadb.model.Process;

public class BackupInstancesAction extends AbstractAction {
	private String _processo = null;
	private String _version;
	private String _dataInizio= null;
	private String _dataFine = null;
	
	private boolean _backupData = false;
	private boolean _deleteData = false;
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		
		org.spagic.components.jobs.backup.parameters.ParametersType backupParametersType = new org.spagic.components.jobs.backup.parameters.ParametersType();
		org.spagic.components.jobs.delete.parameters.ParametersType deleteParametersType = new org.spagic.components.jobs.delete.parameters.ParametersType();
		
		if(_processo != null && Long.parseLong(_processo)!=Constants.ALL_PROCESS){
			Process process = ProcessBO.getProcessDetail(Long.parseLong(_processo));
			backupParametersType.setProcessName(process.getName());
			deleteParametersType.setProcessName(process.getName());
		}
		if(_version != null) {
			backupParametersType.setProcessVersion(_version);
			deleteParametersType.setProcessVersion(_version);
		}
		if(_dataInizio != null) {
			 backupParametersType.setFromDate(_dataInizio);
			 deleteParametersType.setFromDate(_dataInizio);
		}
		if(_dataFine != null) {
			backupParametersType.setToDate(_dataFine);
			deleteParametersType.setToDate(_dataFine);
		}
		
		Object response = null;
		Document document = null;
		DOMReader xmlReader = new DOMReader();
		//call web Services to backup 
		if(_backupData) {
			BckDynDataStart_v_200Proxy backupProxy = new BckDynDataStart_v_200Proxy();
			String locationBackupWS = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.locationBackupWS");
			backupProxy.setEndpoint(locationBackupWS);
			try {
				response = backupProxy.run(backupParametersType);
				document = xmlReader.read((org.w3c.dom.Document)response);
				if (document.asXML().contains("<OK>")) 
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10006", "properties.console.messages"));
				else
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10007", "properties.console.messages"));
			} catch(RemoteException re) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "BackupInstancesAction::service:: backupdata", re);
				getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10004", "properties.console.messages"));
				return;
			}
		}
		if(_deleteData) {
			DelDynDataStart_v_200Proxy deleteProxy = new DelDynDataStart_v_200Proxy();
			String locationDeleteWS = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.locationDeleteWS");
			deleteProxy.setEndpoint(locationDeleteWS);
			try {
				response = deleteProxy.run(deleteParametersType);
				document = xmlReader.read((org.w3c.dom.Document)response);
				if (document.asXML().contains("<OK>")) 
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10008", "properties.console.messages"));
				else
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10009", "properties.console.messages"));
			} catch(RemoteException re) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "BackupInstancesAction::service:: deletedata", re);
				getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10005", "properties.console.messages"));
			}
		}
		
	}
	
	private void loadParametersFromRequest(SourceBean request) {
		if(request.getAttribute(Constants.LBL_BACKUP).equals("backup"))
			_backupData = true;
		else
			_deleteData = true;
		_processo = (String)request.getAttribute(Constants.LBL_PROCESS_ID);
		_version = (String)request.getAttribute(Constants.LBL_VERSION);
		_dataInizio = (String)request.getAttribute(Constants.LBL_DATA_INIZIO);
		_dataFine = (String) request.getAttribute(Constants.LBL_DATA_FINE);
	}
}
