/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.Step;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.CriteriaHibernatePage;
import it.eng.console.hibernate.pagination.QueryHibernatePage;
import it.eng.console.util.Constants;
import it.eng.console.util.DateFormatter;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

public class BindingComponentListAction extends ListAction {
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		Query bindingComponentQuery = ProcessInstanceBO.getBindingComponentList();
		QueryHibernatePage hibernatePage = null;
		int page = getRequestPage(serviceRequest, getRequestContainer().getSessionContainer(), Constants.PAGE_BINDING_COMPONENTS);
		hibernatePage = new QueryHibernatePage(bindingComponentQuery, page, Constants.sizeBindingComponentList);
		List bindingComponentList = hibernatePage.getList();
		
		//create response
		Object[] stepAsList = null;
		SourceBean bindingComponentBeanList = new SourceBean("BINDING_COMPONENT_LIST");
		for(int index=0; index<bindingComponentList.size(); index++) {
			stepAsList = (Object[])bindingComponentList.get(index);
			bindingComponentBeanList.setAttribute( createBindingComponentBean(stepAsList));
		}
		serviceResponse.setAttribute(bindingComponentBeanList);
	}
	
	
	private SourceBean createBindingComponentBean(Object[] stepAsList) 
												throws SourceBeanException {		
		SourceBean bindingComponent = null;
		boolean isStart, isEnd;
		try {
			bindingComponent = new SourceBean("BINDING_COMPONENT");
			bindingComponent.setAttribute("name", stepAsList[1]);
			try {
				bindingComponent.setAttribute("id", ((BigInteger)stepAsList[0]).longValue());
				bindingComponent.setAttribute("received", ((stepAsList[4]!= null) ? ((BigInteger)stepAsList[4]).longValue() : new Long(0)));
				bindingComponent.setAttribute("sent", ((stepAsList[5]!= null) ? ((BigInteger)stepAsList[5]).longValue() : new Long(0)));
				isStart = (Boolean)stepAsList[2];
				isEnd = (Boolean)stepAsList[3];
				bindingComponent.setAttribute("error", ((stepAsList[7]!= null) ? ((BigInteger)stepAsList[7]).longValue() : new Long(0)));
			}catch(ClassCastException cce){
				bindingComponent.setAttribute("id", ((BigDecimal)stepAsList[0]).longValue());
				bindingComponent.setAttribute("received", ((stepAsList[4]!= null) ? ((BigDecimal)stepAsList[4]).longValue() : new Long(0)));
				bindingComponent.setAttribute("sent", ((stepAsList[5]!= null) ? ((BigDecimal)stepAsList[5]).longValue() : new Long(0)));
				isStart = (((Character)stepAsList[2]).toString().equalsIgnoreCase("1")) ? true : false;
				isEnd = (((Character)stepAsList[3]).toString().equalsIgnoreCase("1")) ? true : false;
				bindingComponent.setAttribute("error", ((stepAsList[7]!= null) ? ((BigDecimal)stepAsList[7]).longValue() : new Long(0)));
			}
			
			if (isStart && isEnd)
				bindingComponent.setAttribute("waiting", new Long(0));
			else {
				try {
					bindingComponent.setAttribute("waiting",  ((stepAsList[6]!= null) ? ((BigInteger)stepAsList[6]).longValue() : new Long(0)));
				}catch(ClassCastException cce){
					bindingComponent.setAttribute("waiting",  ((stepAsList[6]!= null) ? ((BigDecimal)stepAsList[6]).longValue() : new Long(0)));
				}
			}
			
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessListAction::createProcessBean:: ", sbe);
			throw sbe;
		}
		
		return bindingComponent;
	}
	
}
