/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.module;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.QueryHibernatePage;
import it.eng.console.process.bo.QueryBean;
import it.eng.console.query.bo.QueryBO;
import it.eng.console.util.Constants;
import it.eng.console.util.DateFormatter;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;
import org.spagic.metadb.model.SearchQuery;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

public class ProcessInstanceListReExecuteModule extends ProcessInstanceListFilterModule {
	public static final String CORRELATION_ID_PREFIX = "RESTARTED_ID:";
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		_page = getRequestPage(serviceRequest, sessionContainer);
		SourceBean processBean = new SourceBean("LIST");

	    //retrieve from db serialized object
		//leggo il parametro queryId 
		String queryId = (String)serviceRequest.getAttribute(Constants.LBL_ID_QUERY);
		String fromHome = (String)serviceRequest.getAttribute(Constants.LBL_FROM_HOME);
		if(fromHome!=null && fromHome.equals("true")){
			List<SearchQuery> queryList= null;	
			serviceResponse.setAttribute(Constants.LBL_FROM_HOME,"true");
			// devo mettere a disposizione della jsp anche la lista delle query preferite
			try {
				queryList = loadQuery(serviceResponse);
			}catch(Exception e){
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListReExecuteModule::loadQuery: ", e);
			}
			
			// arrivo dalla home
			if(queryId!=null && !queryId.equals("")){
				//nel cookie  impostata la query di default verifico se esiste ancora su db
				boolean found=false;
				if(queryList!=null && queryList.size()>0){
					for(int i=0;i<queryList.size();i++){
						SearchQuery searchObj = (SearchQuery)queryList.get(i);
						if(searchObj.getIdQuery().equals(new Long(queryId))){
							found=true;
						}
					}
				}
				if(!found){
					queryId=null;
					TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListReExecuteModule::loadQuery: L'id settato nel cookie non  pi presente su db");
					System.out.println(" L'id settato nel cookie non  pi presente su db");
				}
			}
			if(queryId==null || queryId.equals("")){
				// queryId non definita nel cookie allora recupero la pi recente salvata su db
				Long id = QueryBO.getLastQuery();
				if(id==null){
					//Non ci sono record sulla base dati
					return;
				}
				queryId =id.toString();
			}
		}

		String beanSerialized = QueryBO.getQuery(new Long(queryId));
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("bean", QueryBean.class);
		QueryBean bean = (QueryBean)xstream.fromXML(beanSerialized);

		String queryStr = bean.getQueryStr();
		
		HashMap<String, Object> parMap = bean.getParametersMap(); 
		String type = (String)parMap.get("type");
		Long idPoolType = (Long)parMap.get("idPoolType");
		Date start =(Date)parMap.get("start_date");
		Date end =(Date)parMap.get("end_date");
		String org = (String)parMap.get("org");
		Query query = ProcessInstanceBO.getProcessIstanceList(queryStr, type, idPoolType.longValue(), start, end, org);
		//pagination
		QueryHibernatePage hibernatePage = null;
		hibernatePage = new QueryHibernatePage(query, _page, Constants.sizeProcessInstanceListFiltered);
		List processInstanceList = hibernatePage.getList();

		
		//create response
		Iterator processIt = processInstanceList.iterator();
		List processInstance = null;
		while (processIt.hasNext()) {
			processInstance = (List)processIt.next();
			processBean.setAttribute(createProcessInstanceBean(processInstance, sessionContainer));					
		} 
		// serve per la paginazione per dire che il modulo da rieseguire  sempre lo stesso 
		
		serviceResponse.setAttribute(Constants.LBL_ID_QUERY,queryId);
		serviceResponse.setAttribute("reExecuteQuery",true);
		
		serviceResponse.setAttribute(processBean);
	}
	
	private List<SearchQuery> loadQuery(SourceBean serviceResponse) throws Exception{
		SourceBean queryBeanList = new SourceBean("LISTQUERY");
		
		List<SearchQuery> queryList = QueryBO.getQueryList();
		//create response
		SearchQuery query = null;
		for(int indexQuery=0; indexQuery<queryList.size(); indexQuery++) {
			query = (SearchQuery)queryList.get(indexQuery);
			if (query!=null) {
				queryBeanList.setAttribute(createQueryBean(query));
			}
		}
		serviceResponse.setAttribute(queryBeanList);
		return queryList;

	}
	protected SourceBean createQueryBean(SearchQuery query) throws SourceBeanException {		
		SourceBean item = null;
		long idProcess;
		try {
			item = new SourceBean("ITEM");
			item.setAttribute("id_query", query.getIdQuery());
			item.setAttribute("label", query.getLabel());				
			item.setAttribute("query", query.getQuery());
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessSearchAction::createQueryBean:: ", sbe);
			throw sbe;
		}
		return item;
	}
	
	private int getRequestPage(SourceBean request, SessionContainer session) {
		String pageFromRequest = (String)request.getAttribute(Constants.PAGE_CMD);
		int page = 0;
		if(pageFromRequest == null || pageFromRequest.equalsIgnoreCase(Constants.FIRST_PAGE))
			page = 0;
		else if(pageFromRequest.equalsIgnoreCase(Constants.PREV_PAGE)) {
			page = ((Integer)session.getAttribute(Constants.PAGE_INSTANCES)).intValue() - 1;
			page = (page < 0) ? 0 : page;
		} 
		else if(pageFromRequest.equalsIgnoreCase(Constants.NEXT_PAGE)) 
			page = ((Integer)session.getAttribute(Constants.PAGE_INSTANCES)).intValue() + 1;
		else if(pageFromRequest.equalsIgnoreCase(Constants.LAST_PAGE)) 
			page = Constants.LAST_PAGE_VALUE;
		else if(pageFromRequest.equalsIgnoreCase(Constants.CURRENT_PAGE))
			page = ((Integer)session.getAttribute(Constants.PAGE_INSTANCES)).intValue();
		
		session.setAttribute(Constants.PAGE_INSTANCES, page);
		return page;
	}

	
	protected SourceBean createProcessInstanceBean(List processInstance, SessionContainer sessionContainer) 
	throws SourceBeanException {		
		SourceBean itemBean = null;		
		String iter, owner, iterIstanza;
		try {
			itemBean = new SourceBean("ITEM");
			itemBean.setAttribute("id", processInstance.get(0));
			itemBean.setAttribute("idProcesso", processInstance.get(1));
			itemBean.setAttribute("processo", processInstance.get(2));
			iter = (processInstance.get(3)!=null) ? (String)processInstance.get(3) : "";
			itemBean.setAttribute("iter", iter);	
			iterIstanza = (processInstance.get(4)!=null) ? (String)processInstance.get(4) : "";
			itemBean.setAttribute("iterIstanza", decodInstanceIter(iterIstanza));	
			owner = (processInstance.get(5)!=null) ? (String)processInstance.get(5) : "";
			itemBean.setAttribute("owner", owner);
			
			DateFormat formatter = DateFormatter.getShortDateTimeFormat(sessionContainer);
			String dateTemp = formatter.format(processInstance.get(6));					
			itemBean.setAttribute("dataInizio", dateTemp);				
			dateTemp = (processInstance.get(7)!=null) ? formatter.format(processInstance.get(7)) : "";					
			itemBean.setAttribute("dataFine", dateTemp);
										
			itemBean.setAttribute("stato",processInstance.get(8));
			itemBean.setAttribute("restarted",((String)(processInstance.get(9))).contains(CORRELATION_ID_PREFIX) ? "TRUE" : "FALSE");
	
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredByParametersAction::createProcessInstanceFilteredBean:: ", sbe);
			throw sbe;
		}
		return itemBean;
	}
	
	private String decodInstanceIter(String uniquePoolInstanceKey) {
		char open = '[';
		char closed = ']';
		char sep = ':';
		char blank = ' ';
		int indexOpen = 0;
		int indexSep = 0;
		int indexClosed = 0;
		String idAttribute;
		String decodInstanceIter = "";
		
		indexOpen = uniquePoolInstanceKey.indexOf(open);
		while(indexOpen != -1) {
			indexSep = uniquePoolInstanceKey.indexOf(sep);
			idAttribute = uniquePoolInstanceKey.substring(indexOpen+1, indexSep);
			decodInstanceIter += ProcessInstanceBO.getAttributeName(Long.parseLong(idAttribute)) + sep;
			indexClosed = uniquePoolInstanceKey.indexOf(closed);
			decodInstanceIter += uniquePoolInstanceKey.substring(indexSep+1, indexClosed) + blank;
			uniquePoolInstanceKey = uniquePoolInstanceKey.substring(indexClosed+1);
			indexOpen = uniquePoolInstanceKey.indexOf(open);
		}	
		
		return decodInstanceIter;
	}
}