/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.module;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.util.Constants;
import it.eng.console.util.DateFormatter;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spagosoa.smx.components.restart.client.RestartResult;
import it.eng.spagosoa.smx.components.restart.client.RestartServiceProxy;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class RestartAllInstancesModule extends ProcessInstanceListFilteredByParametersModule {

	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		//get Parameters from request
		SourceBean paginationInfo = (SourceBean)serviceRequest.getAttribute("PAGINATION_INFO");
		loadParametersFromRequest(paginationInfo);
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		//save selected or deseselected processes to restart
		if(_restart != null && _restart.equalsIgnoreCase("TRUE")) 
			saveDifferentBehaviourToRestart(serviceRequest, sessionContainer);
		// recupero (se ci sono) parametri legati all'iter e all'istanza
		List<SourceBeanAttribute> instancePoolInputParameters = getInstancePoolInput(paginationInfo);
		//recupero (se ci sono) attributi rilevanti di processo 
		List<SourceBeanAttribute> relevantAttributeProcessList = getAttributeProcessInput(paginationInfo);

		//recupero gli id dei processi che effettivamente devono essere rilanciati
		//e preparo input per WebService
		Date start = DateFormatter.toDate(_dataInizio, sessionContainer);
		Date end = DateFormatter.toDate(_dataFine, sessionContainer);
		long[] idRestartList  = null;
		ArrayList differentBehaviourToRestart = (ArrayList)sessionContainer.getAttribute(Constants.DIFFERENT_BEHAVIOUR);
		if (!(Boolean)sessionContainer.getAttribute(Constants.SELECT_ALL)) {
			idRestartList = new long[differentBehaviourToRestart.size()];
			for (int index = 0; index<differentBehaviourToRestart.size(); index++) 
				idRestartList[index] = (Long)differentBehaviourToRestart.get(index);
		} else {
			long processIdTemp = 0;
			List processInstanceIdList = ProcessInstanceBO.getProcessInstanceIdList(Long.parseLong(_processo), (new Long(_idPoolType)).longValue(), _stato, start, end, _owner, _ordinamento, _tipoOrdinamento, instancePoolInputParameters, relevantAttributeProcessList);
			int idRestartSize = processInstanceIdList.size();
			idRestartList = new long[idRestartSize - differentBehaviourToRestart.size()];
			int indexEventIdRestart;
			int indexIdRestart = 0;
			for (indexEventIdRestart=0; indexEventIdRestart<idRestartSize; indexEventIdRestart++){
				processIdTemp = (Long)processInstanceIdList.get(indexEventIdRestart);
				if (!differentBehaviourToRestart.contains(processIdTemp))
					idRestartList[indexIdRestart++] = processIdTemp;
			}
		}	
		//call method to restart all Instances
		RestartServiceProxy proxy = new RestartServiceProxy();
		String locationRestartWS = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.locationRestartWS");
		proxy.setEndpoint(locationRestartWS);
		RestartResult[] restartResult = null;
		try {
			// Restart process
			restartResult = proxy.restartMultipleInstances(idRestartList);
		} catch(RemoteException re) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RestartAllInstancesModule::service:: ", re);
			EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, "10001", "properties.console.messages");
			getErrorHandler().addError(warning);
			return;
		}
		
		serviceResponse.setAttribute("idOriginal",idRestartList);
		serviceResponse.setAttribute("idRestarted",restartResult);
		
	}

}
