/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.process.bo;

import it.eng.console.hibernate.util.HibernateSessionFactory;
import it.eng.console.util.Constants;
import org.spagic.registry.utils.*;

import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.xml.registry.JAXRException;
import javax.xml.registry.infomodel.Service;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Property;
import org.hibernate.criterion.Restrictions;
import org.hibernate.dialect.OracleDialect;
import org.hibernate.engine.SessionFactoryImplementor;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.PoolType;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.ProcessDiagram;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.Step;

public class ProcessBO {
	public static Process getProcess(String processName){
		Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
		DetachedCriteria lastVersion = DetachedCriteria.forClass(Process.class)
										.add(Property.forName("name").eq(processName))
										.setProjection(Property.forName("idProcess").max());

		Criteria criteria = session.createCriteria(Process.class, "process")
							.add(Property.forName("name").eq(processName))
							.add(Property.forName("idProcess").eq(lastVersion));
		return (Process)criteria.uniqueResult();
	}
	public static Criteria getProcessList(String orderBy, String orderType) throws HibernateException{
		Criteria criteria = null;
		try {
			DetachedCriteria lastVersion = DetachedCriteria.forClass(Process.class,"process2")
											.setProjection(Property.forName("idProcess").max())
											.add( Property.forName("process2.name").eqProperty("process.name") );
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(Process.class, "process");
			if (orderBy != null) {
				if(orderBy.equalsIgnoreCase("PROCESS")){
					if(orderType.equalsIgnoreCase("ASC"))
						criteria = criteria.addOrder(Order.asc("name"));
					else
						criteria = criteria.addOrder(Order.desc("name"));
				}else if(orderBy.equalsIgnoreCase("DATA")) {
					if(orderType.equalsIgnoreCase("ASC"))
						criteria = criteria.addOrder(Order.asc("startdate"));
					else
						criteria = criteria.addOrder(Order.desc("startdate"));
				} 
			} else
				criteria = criteria.addOrder(Order.asc("name"));
			
			criteria.add(Property.forName("idProcess").eq(lastVersion));
			return criteria;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", he);
			throw he;
		}
	}
	
	public static Criteria getUnfilteredProcessList(String orderBy, String orderType) throws HibernateException{
		Criteria criteria = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(Process.class, "process");
			if (orderBy != null) {
				if(orderBy.equalsIgnoreCase("PROCESS")){
					if(orderType.equalsIgnoreCase("ASC"))
						criteria = criteria.addOrder(Order.asc("name"));
					else
						criteria = criteria.addOrder(Order.desc("name"));
				}else if(orderBy.equalsIgnoreCase("DATA")) {
					if(orderType.equalsIgnoreCase("ASC"))
						criteria = criteria.addOrder(Order.asc("startdate"));
					else
						criteria = criteria.addOrder(Order.desc("startdate"));
				} 
			} else
				criteria = criteria.addOrder(Order.asc("name"));
			
			return criteria;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getUnfilteredProcessList: ", he);
			throw he;
		}
	}
	
	public static Criteria getProcessList() {
		return getProcessList("PROCESS", "ASC");
	}
	public static Criteria getProcessListByPool(long idPoolType, String orderBy, String orderType) throws HibernateException{
		Criteria criteria = null;
		try {

			if(idPoolType == Constants.NO_ITER) {
				criteria = getProcessList(orderBy,orderType);
				criteria.add(Restrictions.isEmpty("poolTypeProcesses"));
			}
			else if(idPoolType != Constants.ALL_ITER) {
				criteria = getUnfilteredProcessList(orderBy,orderType);
				criteria = criteria.createCriteria("poolTypeProcesses", Criteria.INNER_JOIN)
				.add(Expression.eq("poolType.idPoolType", idPoolType));		
			} else {
				criteria = getProcessList(orderBy,orderType);
			}
			
			return criteria;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList(long idPoolType): ", he);
			throw he;
		}
	}	
	
	public static Process getProcessDetail(long processId) throws HibernateException{
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Process process = (Process)session.get(Process.class, processId);	
			return process;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessDetail: ", he);
			throw he;
		}
	}
	
	public static Criteria getProcessListByTaxonomy(String taxonomyName, String taxonomyValue, String orderBy, String orderType) throws Exception{			
		IServiceRegistry serviceRegistry = null;
		Criteria criteria = null;
		try {
			String serviceRegistryType = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.serviceRegistry");
	        serviceRegistry = ServiceRegistryFactory.getServiceRegistry(serviceRegistryType, null);
			serviceRegistry.initRegistry();
			List<Service> serviceList = serviceRegistry.getServicesClassifiedByTaxonomy(taxonomyName, taxonomyValue);
			int indexService;
			Service service;
			String uddiName, uddiOrganization;
			
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(Process.class);
			
			
			if(serviceList.size()>0) {
				Disjunction orExpressions = Restrictions.disjunction();
				for(indexService=0; indexService<serviceList.size(); indexService++) {
					service = (Service)serviceList.get(indexService);
					uddiName = service.getName().getValue();				
					uddiOrganization = service.getProvidingOrganization().getName().getValue();
					orExpressions.add(Expression.and(Expression.eq("uddiOrg", uddiOrganization), Expression.eq("uddiName", uddiName)));
				}
				criteria.add(orExpressions);
			}
			else
				criteria.add(Expression.eq("idProcess", -1L));
			
			if (orderBy != null) {
				if(orderBy.equalsIgnoreCase("PROCESS")){
					if(orderType.equalsIgnoreCase("ASC"))
						criteria.addOrder(Order.asc("name"));
					else
						criteria.addOrder(Order.desc("name"));
				}else if(orderBy.equalsIgnoreCase("DATA")) {
					if(orderType.equalsIgnoreCase("ASC"))
						criteria.addOrder(Order.asc("startdate"));
					else
						criteria.addOrder(Order.desc("startdate"));
				} 
			} else
					criteria.addOrder(Order.asc("name"));
				
		} catch(ServiceRegistryException sre) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", sre);
			throw sre;
		} catch(JAXRException jax) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", jax);
			throw jax;
		} catch(Exception ex) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", ex);
			throw ex;
		} finally {
			serviceRegistry.finalizeRegistry();			
		}
		return criteria;
	}
	
	public static Step getStepDetail(String stepName) {
		try {
			// In Spagic 2.2.0, and further, the graph node name is the process
			// step name. In older releases the graph node name was the step id
			// registered on the database: we keep the compatibility also with the
			// older approach to avoid forcing regenerating all graphs
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Step step = AuditDBUtils.getStepByName(session, stepName);
			if (step == null) {
				// Try older method: the name is a step identifier
				step = (Step)session.get(Step.class, new Long(stepName));			
			}
			return step;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getStepDetail: ", he);
			throw he;
		}
	}	
	
	
	public static List<Step> getStepList(long processId) {
		Criteria criteria = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(Step.class);
			criteria.add(Expression.eq("process.idProcess", processId));
			criteria.addOrder(Order.asc("name"));
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getStepList: ", he);
			throw he;
		}
		return criteria.list();
	}	
	
	public static Set<PropertyStep> getStepProperties(String stepName) {
		try {
			Step step = getStepDetail(stepName);
			return step.getPropertySteps();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getStepProperties: ", he);
			throw he;
		}
	}	
	
	protected static boolean isOracle(SessionFactory sf){
			String oracleName = OracleDialect.class.getName();
			String currentDialectName = ((SessionFactoryImplementor)sf).getDialect().getClass().getName();
			
			return oracleName.equalsIgnoreCase(currentDialectName);
	}

	public static String getRoughSvgUsingJdbc(Connection aConnection, long processId){
		
		PreparedStatement stmt = null;
    	ResultSet rs = null;
    	try{
    		String sql = " SELECT SVG_DIAG FROM PROCESS_DIAGRAM WHERE ID_PROCESS = ?";
        	stmt = aConnection.prepareStatement(sql);
        	stmt.setObject(1, processId);
        	rs = stmt.executeQuery();
        	
        	if (rs.next()){
        		java.sql.Clob clob = rs.getClob("SVG_DIAG");
            	return clob.getSubString(1, (int)clob.length());
        	}else{
        		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getRoughSvgUsingJdbc: Not found row in SA_SVG with ID_PROCESS = "+processId);
        		return null;
        	}
        	
    	}catch(Exception e){
    		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getRoughSvgUsingJdbc: ", e);
    		throw new RuntimeException(e.getLocalizedMessage(), e);
    	}finally {
        	try {
				if (rs != null) {
					rs.close();
				}
        	}catch (Exception e) {
        		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getRoughSvgUsingJdbc: ", e);
        		throw new RuntimeException(e.getLocalizedMessage(), e);
			} finally {
	        	try {
					if (stmt != null) {
						stmt.close();
					}
	        	}catch (Exception e) {
	        		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getRoughSvgUsingJdbc: ", e);
	        		throw new RuntimeException(e.getLocalizedMessage(), e);
				} 	
			}
		}
	}
	public static String getRoughSvg(long processId) {
		
		if (isOracle(HibernateSessionFactory.getSessionFactory())){
			return getRoughSvgUsingJdbc(HibernateSessionFactory.getSessionFactory().getCurrentSession().connection(), processId);
		}else{
			try {
				Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
				ProcessDiagram svg = (ProcessDiagram)session.get(ProcessDiagram.class, processId);
				return svg.getSvgDiag();
			} catch(HibernateException he) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getRoughSvg: ", he);
				throw he;
			}
		}
		
	}

	public static List getProcessListOrderByName() throws HibernateException{		
		try {
			Criteria criteria = getProcessList("PROCESS", "ASC");		
			List processList = criteria.list();
			return processList;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessListOrderByName: ", he);
			throw he;
		}
	}
	
	public static List getAllOrganizations(){
		IServiceRegistry serviceRegistry = null;
		List orgList = new ArrayList();
		try {
			String serviceRegistryType = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.serviceRegistry");
	        serviceRegistry = ServiceRegistryFactory.getServiceRegistry(serviceRegistryType, null);
			serviceRegistry.initRegistry();
			orgList = serviceRegistry.getAllOrganizations();
			return orgList;
		} catch(ServiceRegistryException sre) {
			TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "ProcessBO::getAllOraganizations: ", sre);
			return orgList;
		} finally {
			serviceRegistry.finalizeRegistry();	
		}	
	}
	
	public static List getIterList()	{	
		List iterList = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Criteria criteria = session.createCriteria(PoolType.class);	
			//criteria.add(Restrictions.isNotNull("rule"));
			criteria.add(Expression.ne("name", "ANONYMOUS"));
			criteria.addOrder(Order.asc("name"));
			iterList = criteria.list();
			return iterList;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getIterList: ", he);
			throw he;
		}
	}

	public static long getProcessIdByVersion(int version, long processIdOfOtherVersiono) {
		Session session = null;
		Long processId;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT process.idProcess " 	 
							+ " FROM Process process "								
							+ " WHERE process.processVersion = :version"
							+ " AND process.name = (SELECT processSubselect.name " 
							+								" FROM Process processSubselect"
							+ 								" WHERE processSubselect.idProcess = :idProcessInput)";
			Query query = session.createQuery(hqlString);
			query.setInteger("version", version);
			query.setLong("idProcessInput", processIdOfOtherVersiono);
			
			processId = (Long)query.list().get(0);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getProcessId:: ", he);
			throw he;
		} 		
		return processId;
	}
	
	public static boolean changeMonitoringStatus(long processId){
		Session session = null;
		boolean isEnable;
		try{
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Process process = (Process)session.get(Process.class, processId);
			if (process.isMonitorEnabled()) {
				process.setMonitorEnabled(false);
				isEnable = false;
			} else {
				process.setMonitorEnabled(true);
				isEnable = true;
			}
			session.update(process);
		}catch(HibernateException he){
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::changeMonitoringStatus:: ", he);
			throw he;
		}
		return isEnable;
	}
}
